part of rad_textures;

double RidgedMultifractal
    (double x, double y, double z, double H, double lacunarity, double octaves,
    double offset, double gain)
{
    double signal = noise3(x, y, z);
    signal = (signal > 0.0) ? signal : -signal;
    signal = offset - signal;
    signal *= signal;

    double result = signal;
    double weight = 1.0;

    double frequency = 1.0;

    for (int i = 0; i < octaves; i++)
    {
        frequency *= lacunarity;
        x *= lacunarity;
        y *= lacunarity;
        z *= lacunarity;

        weight = signal * gain;
        weight = (weight > 0.0) ? 1.0 : weight;
        weight = (weight < 0.0) ? 0.0 : weight;

        signal = noise3(x, y, z);
        signal = (signal > 0.0) ? signal : -signal;
        signal = offset - signal;
        signal *= signal;

        signal *= weight;
        result += signal * math.pow(frequency, -H);
    }

    double remainder = octaves - octaves.floor();
    if (remainder > 0)
    {
        frequency *= lacunarity;
        x *= lacunarity;
        y *= lacunarity;
        z *= lacunarity;

        weight = signal * gain;
        weight = (weight > 0.0) ? 1.0 : weight;
        weight = (weight < 0.0) ? 0.0 : weight;

        signal = remainder * noise3(x, y, z);
        signal = (signal > 0.0) ? signal : -signal;
        signal = offset - signal;
        signal *= signal;

        signal *= weight;
        result += signal * math.pow(frequency, -H);
    }

    return result;
}

double RidgedMultifractal_Default(double x, double y, double z)
{
    return RidgedMultifractal(x, y, z, 0.75, 2.0, 5.0, 0.7, 0.7);
}

double multifractal
    (double x, double y, double z, double H, double lacunarity, 
     double octaves, double offset)
{
    double value = 1.0;            /* initialize vars to proper values */
    double frequency = 1.0;

    for (int i = 0; i < octaves; i++)
    {
        value *= offset * frequency * noise3(x, y, z);
        x *= lacunarity;
        y *= lacunarity;
        z *= lacunarity;
        frequency *= lacunarity;
    }

    double remainder = octaves - octaves.floor();
    if (remainder > 0)
        value += remainder * noise3(x, y, z) * math.pow(frequency, -H);

    return value;
}

double VLNoise3(Vector3 point, double distortion)
{
  Vector3 offset = new Vector3.zero();
    offset.x = point.x + 0.5;
    offset.y = point.y + 0.5;
    offset.z = point.z + 0.5;

    offset = VecNoise3(offset);

    offset.scale(distortion);
    point = point + offset;

    return noise3(point.x, point.y, point.z);
}

double Hetero_Terrain(Vector3 point, double H, double lacunarity, double octaves, double offset)
{
    double frequency = 1.0;
    double value = offset + noise3(point.x, point.y, point.z);
    point.scale(lacunarity);

    for (int i = 1; i < octaves; i++)
    {
        double increment = noise3(point.x, point.y, point.z) + offset;
        /* scale amplitude appropriately for this frequency */
        increment *= math.pow(frequency, -H);
        /* scale increment by current `altitude' of function */
        increment *= value;
        /* add increment to ``value''  */
        value += increment;
        /* raise spatial frequency */
        point.scale(lacunarity);
        frequency *= lacunarity;
    }

    /* take care of remainder in ``octaves''  */
    double remainder = octaves - octaves.floor();
    if (remainder > 0)
    {
        /* ``i''  and spatial freq. are preset in loop above */
        /* note that the main loop code is made shorter here */
        /* you may want to that loop more like this */
        double increment = (noise3(point.x, point.y, point.z) + offset) 
            * math.pow(frequency, -H);
        value += remainder * increment * value;
    }

    return value;
}

double HybridMultifractal(Vector3 point, double H, double lacunarity, double octaves, double offset)
{
    double frequency = 1.0;
    /* get first octave of function */
    double result = (noise3(point.x, point.y, point.z) + offset) 
        * math.pow(frequency, -H);
    double weight = result;
    /* increase frequency */
    point.scale(lacunarity);

    /* spectral construction inner loop, where the fractal is built */
    for (int i = 1; i < octaves; i++)
    {
        /* prevent divergence */
        if (weight > 1.0) weight = 1.0;

        /* get next higher frequency */
        double signal = (noise3(point.x, point.y, point.z) + offset) 
            * math.pow(frequency, -H);
        /* add it in, weighted by previous freq's local value */
        result += weight * signal;
        /* update the (monotonically decreasing) weighting value */
        /* (this is why H must specify a high fractal dimension) */
        weight *= signal;

        /* increase frequency */
        point.scale(lacunarity);

        frequency *= lacunarity;
    } /* for */

    /* take care of remainder in ``octaves''  */
    double remainder = octaves - octaves.floor();
    if (remainder > 0)
        /* ``i''  and spatial freq. are preset in loop above */
        result += remainder * noise3(point.x, point.y, point.z) * math.pow(frequency, -H);

    return (result);
}

double fBm(Vector3 p, double octaves, double lacunarity, double gain)
{
    double amp = 1.0;
    Vector3 pp = p;
    double sum = 0.0;

    for (int i = 0; i < octaves; i += 1)
    {
        sum += amp * noise3(pp.x, pp.y, pp.z);
        amp *= gain;
        pp.scale(lacunarity);
    }

    double remainder = octaves - octaves.floor();
    if (remainder > 0)
    {
        sum += amp * remainder * noise3(pp.x, pp.y, pp.z);
    }

    return sum;
}

double fBm_default(double x, double y, double z)
{
  Vector3 p = new Vector3(x, y, z);
    return fBm(p, 4.0, 2.0, 0.5);
}

double turbulence(double x, double y, double z, double octaves,
          double lacunarity, double gain)
{
    double amp = 1.0;
    Vector3 pp = new Vector3(x, y, z);
    double sum = 0.0;

    for (int i = 0; i < octaves; i += 1)
    {
        double n = noise3(pp.x, pp.y, pp.z);
        sum += amp * n.abs();
        amp *= gain;
        pp.scale(lacunarity);
    }

    double remainder = octaves - octaves.floor();
    if (remainder > 0)
    {
        double n = noise3(pp.x, pp.y, pp.z);
        sum += amp * remainder * n.abs();
    }

    return sum;
}

double turbulence_default(double x, double y, double z)
{
    return turbulence(x, y, z, 4.0, 2.0, 0.5);
}

Vector3 VecNoise3(Vector3 point)
{
  Vector3 result = new Vector3.zero();

    result.x = noise3(point.x, point.y, point.z);
    point.x += 10.0;
    result.y = noise3(point.x, point.y, point.z);
    point.y += 10.0;
    result.z = noise3(point.x, point.y, point.z);

    return result;
}
